@@ -8,6 +8,10 @@ FeatureUtil
 Authors: wangkai02(wangkai02@corp.netease.com)
 Phone: 17816029211
 Date: 2019/9/11
+
+Authors: zouzhene(zouzhene@corp.netease.com)
+Phone: 13261632788
+Date: 2020/3/11
 """
 from tensorflow.python.data.ops import dataset_ops
 import tensorflow as tf
@@ -17,7 +21,18 @@ import scipy
 from tensorflow.python.keras.preprocessing import sequence
 
 
-class FeatureUtil():
+class FeatureUtil(object):
+    '''
+    构建数据集的工具类，构建的数据集以本地文件的方式进行存储。
+    目前支持两种数据存储格式：tfrecord 和 hdf5。
+
+    Args:
+        config: 配置文件，定义了如下一些属性：
+                    序列数量、
+
+                    序列最大长度
+    '''
+
     def __init__(self, config):
         self.config = config
         self.maxlen = config['maxlen']
@@ -31,26 +46,35 @@ class FeatureUtil():
         self.neg_sample = config['neg_sample']
         self.seq_num = self.config['seq_num']
 
-    def feature_extraction(self, data, predict=False, padding=False, sparse=True,torch=False):
+    def feature_extraction(self, data, serving=False, write_to='hdf5'):
+        '''
+        从原始数据中提取特征，后续调用不同的方法将特征存储为 tfrecord 或者 hdf5 格式。
+
+        Args:
+            data: 原始数据
+            predict: 是否预测阶段
+            write_to: 'tfrecord' or 'hdf5'， 'tfrecord' default
+
+        Returns:
+            特征数据
+        '''
         feature_id = {
             'role_id': 0,
             'role_id_hash': 1,
             'sequence_feature': 2,
             'cross_feature': 3,
             'user_feature': 4,
-            'pos_label': 5,
-            'neg_label': 6,
-            'cur_time': 7,
-            'weight': 8
+            'label': 5,
+            'cur_time': 6,
+            'weight': 7
         }
 
         role_id = [[int(sample.split('@')[feature_id['role_id']])] for sample in data]
         role_id_hash = [int(sample.split('@')[feature_id['role_id_hash']]) for sample in data]
         # pos_label = [int(sample.split('@')[feature_id['pos_label']].split(':')[0]) for sample in data]
 
-        pos_label = [[x for x in map(int, sample.split('@')[feature_id['pos_label']].split(','))] for sample in data]
-        neg_label = [[x for x in map(int, sample.split('@')[feature_id['neg_label']].split(','))] for sample in data]
-        mask = [[x for x in set(pos_label[i] + neg_label[i])] for i in range(len(data))]
+        mask, label = [[int(x.split(':')[0]) for x in sample.split('@')[feature_id['label']].split(',')] for sample in data], \
+                      [[int(x.split(':')[1]) for x in sample.split('@')[feature_id['label']].split(',')] for sample in data]
 
         cur_time = [int(sample.split('@')[feature_id['cur_time']]) for sample in data]
 
@@ -72,28 +96,78 @@ class FeatureUtil():
         user_features_id = [sample + [0] * (self.user_feature_num - len(sample)) for sample in user_features_id]
         # label_week_id = [[int(data[i].split('@')[feature_id['pos_label']].split(':')[1])] * len(sequence_id[i]) for i in range(len(data))]
 
-        if torch:
+        if serving:
+            # 临时获取少量数据，
+            # 在此处对稀疏向量做了 batch 处理
             cross_features_index = [[[i, int(y.split(':')[0])] for y in re.split(', |,', sample.split('@')[feature_id['cross_feature']])] for i, sample in enumerate(data)]
             cross_features_index, cross_features_val = self.cross_features_batch(cross_features_index, cross_features_val)
+            # 序列特征，padding
             sequence_id = [sequence.pad_sequences(x, maxlen=self.maxlen, padding='post') for x in sequence_id]
             sequence_time = [sequence.pad_sequences(x, maxlen=self.maxlen, padding='post') for x in sequence_time]
             sequence_time_gaps = [sequence.pad_sequences(x, maxlen=self.maxlen, padding='post') for x in sequence_time_gaps]
-            return np.array(role_id_hash), np.array(sequence_id), np.array(sequence_time), np.array(sequence_time_gaps), \
-                   np.array(cross_features_index), np.array(cross_features_val), np.array(user_features_id), np.array(cur_time),np.array(pos_label)
+            # mask、label，处理成 multi-hot
+            mask, label = self.mask_label_multi_hot(mask, label)
 
-        if predict == True:
-            # assert len(sequence_id) == 1
-            cross_features_index = [[[i, int(y.split(':')[0])] for y in re.split(', |,', sample.split('@')[feature_id['cross_feature']])] for i, sample in enumerate(data)]
-            cross_features_index, cross_features_val = self.cross_features_batch(cross_features_index, cross_features_val)
+            return np.array(role_id_hash), np.array(sequence_id), np.array(sequence_time), np.array(sequence_time_gaps), \
+                   np.array(cross_features_index), np.array(cross_features_val), np.array(user_features_id), np.array(cur_time), \
+                   np.array(mask), np.array(label)
+
+        if write_to == 'hdf5':
+            # 生成的特征用于写入hdf5
+            # 稀疏特征，处理成 list of array
+            cross_features_index = [np.array(item, dtype=np.int64) for item in cross_features_index]
+            cross_features_val = [np.array(item, dtype=np.float32) for item in cross_features_val]
+            # 序列特征，padding
             sequence_id = [sequence.pad_sequences(x, maxlen=self.maxlen, padding='post') for x in sequence_id]
             sequence_time = [sequence.pad_sequences(x, maxlen=self.maxlen, padding='post') for x in sequence_time]
             sequence_time_gaps = [sequence.pad_sequences(x, maxlen=self.maxlen, padding='post') for x in sequence_time_gaps]
-            return np.array(role_id_hash), np.array(sequence_id), np.array(sequence_time), np.array(sequence_time_gaps), \
-                   np.array(cross_features_index), np.array(cross_features_val), np.array(user_features_id), np.array(cur_time)
+            # mask、label，处理成 multi-hot
+            mask, label = self.mask_label_multi_hot(mask, label)
 
-        return role_id_hash, sequence_id, sequence_time, sequence_time_gaps, cross_features_index, cross_features_val, user_features_id, pos_label, neg_label, mask, cur_time
+            return np.array(role_id_hash, dtype=np.int64), np.array(sequence_id, dtype=np.int64), \
+                   np.array(sequence_time, dtype=np.int64), np.array(sequence_time_gaps, dtype=np.int64), \
+                   cross_features_index, cross_features_val, \
+                   np.array(user_features_id, dtype=np.int64), np.array(cur_time, dtype=np.int64), \
+                   np.array(mask), np.array(label)
+
+        elif write_to == 'tfrecord':
+            # 生成的特征用于写入tfrecord
+            return role_id_hash, sequence_id, sequence_time, sequence_time_gaps, \
+                   cross_features_index, cross_features_val, user_features_id, \
+                   cur_time, mask, label
+
+        else:
+            raise Exception('feature_extraction get unexpected "write_to", %s not in ["hdf5", "tfrecord"]' % (write_to))
+
+    def mask_label_multi_hot(self, mask, label):
+        """Return the multi-hot of mask and label ids
+
+        Args:
+            mask: mask ids
+            label: label ids
+
+        Returns:
+            multi-hot
+        """
+        mask_tmp = [[0] * self.output_unit for _ in range(len(mask))]
+        label_tmp = [[0] * self.output_unit for _ in range(len(label))]
+        for i, (line_mask, line_label) in enumerate(zip(mask, label)):
+            for item_mask, item_label in zip(line_mask, line_label):
+                mask_tmp[i][item_mask] = 1
+                label_tmp[i][item_mask] = item_label
+        return mask_tmp, label_tmp
 
     def cross_features_batch(self, index, val):
+        """
+        将稀疏特征处理成 batch 的形式，后续用于线上服务推理
+
+        Args:
+            index:稀疏特征的索引
+            val:稀疏特征的值
+
+        Returns:
+            batched cross feature
+        """
         batch_num = len(index)
         index = [b for a in index for b in a]
         val = [b for a in val for b in a]
@@ -114,12 +188,27 @@ class FeatureUtil():
         return index, val
 
     def to_tfrecord(self, csvfile, filename):
+        """Extract features from the original data in csv format, and convert to tfrecord format.
+
+        Note:
+            Tensorflow uses tensorflow.TFRecordDataloader to load tfrecord files
+
+            Pytorch uses pytorch.TFRecordDataloader to load tfrecord files
+
+        Args:
+            csv_file (str): the file, in which original csv data is saved
+            h5_file (str): the file, to which tfrecord data will save
+
+        Returns:
+
+        """
+
         writer = tf.python_io.TFRecordWriter(filename)
         data = open(csvfile, 'r', encoding='utf8').read().split('\n')[1:-1]
         print('feature_extraction')
         for jj in range(1+len(data)//10000):
             role_id_hashs, sequence_ids, sequence_times, sequence_time_gapss, cross_features_indexs, cross_features_vals, user_features_ids, \
-            pos_labels, neg_labels, masks, cur_times = self.feature_extraction(data[jj*10000 : (jj+1)*10000])
+            cur_times, masks, labels = self.feature_extraction(data[jj*10000 : (jj+1)*10000], write_to='tfrecord')
             for i in range(len(role_id_hashs)):
                 role_id_hash = role_id_hashs[i]
                 sequence_id = sequence_ids[i]
@@ -128,8 +217,7 @@ class FeatureUtil():
                 cross_features_index = cross_features_indexs[i]
                 cross_features_val = cross_features_vals[i]
                 user_features_id = user_features_ids[i]
-                pos_label = pos_labels[i]
-                neg_label = neg_labels[i]
+                label = labels[i]
                 mask = masks[i]
                 cur_time = cur_times[i]
 
@@ -141,8 +229,7 @@ class FeatureUtil():
                 cross_features_index_feature = tf.train.Feature(int64_list=tf.train.Int64List(value=cross_features_index))
                 cross_features_val_feature = tf.train.Feature(float_list=tf.train.FloatList(value=cross_features_val))
                 user_features_id_feature = tf.train.Feature(int64_list=tf.train.Int64List(value=user_features_id))
-                pos_label_feature = tf.train.Feature(int64_list=tf.train.Int64List(value=pos_label))
-                neg_label_feature = tf.train.Feature(int64_list=tf.train.Int64List(value=neg_label))
+                label_feature = tf.train.Feature(int64_list=tf.train.Int64List(value=label))
                 mask_feature = tf.train.Feature(int64_list=tf.train.Int64List(value=mask))
                 cur_time_feature = tf.train.Feature(int64_list=tf.train.Int64List(value=[cur_time]))
 
@@ -152,10 +239,9 @@ class FeatureUtil():
                     "cross_features_index": cross_features_index_feature,
                     "cross_features_val": cross_features_val_feature,
                     "user_features_id": user_features_id_feature,
-                    "pos_label": pos_label_feature,
-                    "neg_label": neg_label_feature,
-                    'mask': mask_feature,
                     'cur_time': cur_time_feature,
+                    'mask': mask_feature,
+                    "label": label_feature,
                 }
                 for seq_num_i in range(self.seq_num):
                     feature['sequence_id_' + str(seq_num_i)] = sequence_id_feature[seq_num_i]
@@ -168,100 +254,49 @@ class FeatureUtil():
                 writer.write(seq_example.SerializeToString())
         writer.close()
 
-    def read_tfrecord(self, filename, is_pred=False):
+    def to_hdf5(self, csv_file, h5_file):
+        """Extract features from the original data in csv format, and convert to hdf5 format.
 
-        def _parse_exmp(serial_exmp):
-            context_features = {
-                "role_id_hash": tf.io.FixedLenFeature([], dtype=tf.int64),
-                "cross_features_index": tf.io.VarLenFeature(dtype=tf.int64),
-                "cross_features_val": tf.io.VarLenFeature(dtype=tf.float32),
-                "user_features_id": tf.io.VarLenFeature(dtype=tf.int64),
-                # "label_week_id": tf.io.VarLenFeature(dtype=tf.int64),
-                "pos_label": tf.io.VarLenFeature(dtype=tf.int64),
-                "neg_label": tf.io.VarLenFeature(dtype=tf.int64),
-                "mask": tf.io.VarLenFeature(dtype=tf.int64),
-                "cur_time": tf.io.FixedLenFeature([], dtype=tf.int64),
-            }
+        Note:
+            Tensorflow uses tensorflow.HDF5Dataloader to load hdf5 files
 
-            for seq_num_i in range(self.seq_num):
-                context_features['sequence_id_' + str(seq_num_i)] = tf.io.VarLenFeature(dtype=tf.int64)
-                context_features['sequence_time_' + str(seq_num_i)] = tf.io.VarLenFeature(dtype=tf.int64)
-                context_features['sequence_time_gaps_' + str(seq_num_i)] = tf.io.VarLenFeature(dtype=tf.int64)
+            Pytorch uses pytorch.HDF5Dataloader to load hdf5 files
 
-            context_parsed = tf.io.parse_single_example(serialized=serial_exmp, features=context_features)
+        Args:
+            csv_file (str): the file, in which original csv data is saved
+            h5_file (str): the file, to which hdf5 data will save
 
-            sequence_id = [tf.sparse_to_dense(context_parsed['sequence_id_' + str(i)].indices, [self.maxlen],
-                                              context_parsed['sequence_id_' + str(i)].values) for i in range(self.seq_num)]
-            sequence_time = [tf.sparse_to_dense(context_parsed['sequence_time_' + str(i)].indices, [self.maxlen],
-                                                context_parsed['sequence_time_' + str(i)].values) for i in range(self.seq_num)]
-            sequence_time_gaps = [tf.sparse_to_dense(context_parsed['sequence_time_gaps_' + str(i)].indices, [self.maxlen],
-                                                     context_parsed['sequence_time_gaps_' + str(i)].values)
-                                  for i in range(self.seq_num)]
-            role_id_hash = context_parsed['role_id_hash']
-            cross_features_index = tf.sparse.to_dense(context_parsed['cross_features_index'])
-            cross_features_val = tf.sparse.to_dense(context_parsed['cross_features_val'])
-            cross_feature = tf.SparseTensor(values=cross_features_val, indices=tf.expand_dims(cross_features_index, -1),
-                                            dense_shape=[self.cross_feature_num])
-            user_feature = tf.sparse.to_dense(context_parsed['user_features_id'])
+        Returns:
 
-            pos_label = tf.sparse.to_dense(context_parsed['pos_label'])
-            neg_label = tf.sparse.to_dense(context_parsed['neg_label'])
-            mask = tf.sparse.to_dense(context_parsed['mask'])
-            cur_time = context_parsed['cur_time']
-
-            indices = tf.expand_dims(pos_label, -1)
-            updates = tf.ones_like(pos_label)
-            shape = tf.constant([self.output_unit], dtype='int64')
-            label = tf.scatter_nd(indices, updates, shape)
-
-            if self.multi_class and self.neg_sample:
-                indices = tf.expand_dims(mask, -1)
-                updates = tf.ones_like(mask)
-                shape = tf.constant([self.output_unit], dtype='int64')
-                mask = tf.scatter_nd(indices, updates, shape)
+        """
+        with open(csv_file, 'r', encoding='utf8') as iin:
+            data = iin.read().split('\n')[1:-1]
+            role_id_hash, sequence_id, sequence_time, sequence_time_gaps, cross_features_index, cross_features_val, user_features_id, \
+            cur_time, mask, label = self.feature_extraction(data, write_to='hdf5')
 
+        import h5py
+        with h5py.File(h5_file, 'w') as f:
+            f.create_dataset('role_id_hash', data=np.array(role_id_hash))
+            f.create_dataset('sequence_id', data=np.array(sequence_id))
+            f.create_dataset('sequence_time', data=np.array(sequence_time))
+            f.create_dataset('sequence_time_gaps', data=np.array(sequence_time_gaps))
+            if len(set(len(x) for x in cross_features_index)) == 1:
+                f.create_dataset('cross_features_index', data=cross_features_index)
+                f.create_dataset('cross_features_val', data=cross_features_val)
             else:
-                if self.output_unit > 1:
-                    mask = tf.ones([self.output_unit])
-                else:
-                    mask = tf.ones([1])
-            return role_id_hash, sequence_id, sequence_time, sequence_time_gaps, cross_feature, user_feature, mask, cur_time, label
+                f.create_dataset('cross_features_index', data=cross_features_index, dtype=h5py.special_dtype(vlen=np.dtype('int64')))
+                f.create_dataset('cross_features_val', data=cross_features_val, dtype=h5py.special_dtype(vlen=np.dtype('float32')))
+            f.create_dataset('user_features_id', data=np.array(user_features_id))
+            f.create_dataset('cur_time', data=np.array(cur_time))
+            f.create_dataset('mask', data=np.array(mask))
+            f.create_dataset('label', data=np.array(label))
 
-        def _flat_map_fn(role_id_hash, sequence_id, sequence_time, sequence_time_gaps, cross_feature, user_feature, mask, cur_time, label):
 
-            return dataset_ops.Dataset.zip((
-                role_id_hash.batch(batch_size=self.batchsize),
-                sequence_id.padded_batch(self.batchsize, padded_shapes=([self.seq_num, self.maxlen])),
-                sequence_time.padded_batch(self.batchsize, padded_shapes=([self.seq_num, self.maxlen])),
-                sequence_time_gaps.padded_batch(self.batchsize, padded_shapes=([self.seq_num, self.maxlen])),
-                cross_feature.batch(batch_size=self.batchsize),
-                user_feature.padded_batch(self.batchsize, padded_shapes=([self.user_feature_num])),
-                mask.batch(batch_size=self.batchsize),
-                cur_time.batch(batch_size=self.batchsize),
-                label.batch(batch_size=self.batchsize),
-            ))
+if __name__ == '__main__':
+    from demo.qier.code import param
 
-        def preprocess_fn(a, b, c, d, e, f, g, h, i):
-            '''A transformation function to preprocess raw data
-            into trainable input. '''
-            return (a, b, c, d, e, f, g, h), i
-            # return (a, b, c, d, e, f, g, i), tf.one_hot(h, self.output_unit) if self.output_unit > 1 else h
-
-        # tf.enable_eager_execution()
-        dataset = tf.data.TFRecordDataset(filename, num_parallel_reads=4)
-        if is_pred:
-            dataset_train = dataset \
-                .map(_parse_exmp, num_parallel_calls=1) \
-                .window(size=self.batchsize, drop_remainder=False) \
-                .flat_map(_flat_map_fn) \
-                .map(preprocess_fn)
-        else:
-            dataset_train = dataset \
-                .map(_parse_exmp, num_parallel_calls=4) \
-                .shuffle(10000) \
-                .window(size=self.batchsize, drop_remainder=True) \
-                .flat_map(_flat_map_fn) \
-                .map(preprocess_fn) \
-                .repeat()
-        # print(dataset_train.make_one_shot_iterator().get_next())
-        return dataset_train
+    config = param.config
+    file = '/root/rslib/demo/qier/code/dataset/2019-11-21.age_trainset.csv'
+    tfrecord_file = '/root/rslib/demo/qier/code/dataset/2019-11-21.age_trainset.tfrecord'
+    featureutil = FeatureUtil(config)
+    featureutil.to_hdf5(file, tfrecord_file)
