@@ -50,6 +50,8 @@ class RecEnvSrc(object):
         self.l3_ssr_item = set()
         self.taohua_item = set()
 
+        self.new_item=set()
+
         self.item_id_embedding = [[0] for _ in range(300)]
 
         self.FeatureUtil = FeatureUtil(config)
@@ -59,12 +61,15 @@ class RecEnvSrc(object):
         self.fa = open(actionfile, 'r')
         self.fa.readline()
 
-        self.rawstate_cache(self.fs, 10000)
+        self.rawstate_cache(self.fs, 40000)
         self.action_cache(self.fa)
 
     def rawstate_cache(self, f, num):
-        for i in range(num):
-            tmp = f.readline()
+        lines = f.readlines()
+        llen = min(num, len(lines))
+        for i in range(llen):
+            # tmp = f.readline()
+            tmp = lines[i]
             role_id = tmp.split('@')[0]
             if len(role_id) > 1:
                 self.state_dict[role_id] = tmp
@@ -105,6 +110,8 @@ class RecEnvSrc(object):
                     self.l3_ssr_item.add(item_id_map)
                     self.l0_ssr_mask[item_id_map] = 1
                     self.l0_ssr_item.add(item_id_map)
+                if item_id[2] in ['4','5']:
+                    self.new_item.add(item_id)
 
             # 统计不同支付方式的物品
             t = item_id[1]
@@ -142,7 +149,7 @@ class RecEnvSrc(object):
                 elif t == '3':
                     price *= 4.0
                 else:
-                    price *= 0.1
+                    price *= 0.05
             else:
                 raise Exception('env\'s reward_type is invalid !')
 
@@ -188,7 +195,7 @@ class RecEnvSrc(object):
         return feature
 
     def rawstate_to_state(self, state):
-        feature = self.FeatureUtil.feature_extraction(data=state, predict=True)
+        feature = self.FeatureUtil.feature_extraction(data=state, serving=True)
         return feature
 
     def get_new_user_rawstate(self, rawstate, info):
