@@ -1,7 +1,8 @@
 import gym
 from gym.utils import seeding
 
-from rslib.algo.atrank import ATRankforRL3
+from rslib.algo.tensorflow.atrank import ATRankforRL3
+
 from rslib.core.FeatureUtil import FeatureUtil
 from rslib.gym_envs.l20_mystic.envs.L20RecEnvSrc import L20RecEnvSrc
 from rslib.gym_envs.l20_mystic.envs.L20RecSim import L20RecSim
@@ -51,7 +52,7 @@ class L20RecEnv(gym.Env):
         self.info = info
         # observation用于RL学习，每步都变
         if self.use_rslib_model:
-            return self.FeatureUtil.feature_extraction(data=self.cur_rawstate, predict=True), reward, done, info
+            return self.FeatureUtil.feature_extraction(data=self.cur_rawstate, serving=True), reward, done, info
 
         observation = self.controllerobs.get_obs(new_rawstate, self.info, self.stepp)
 
@@ -70,7 +71,7 @@ class L20RecEnv(gym.Env):
         self.cur_rawstate = self.sim.recEnvSrc.get_user_rawstate(self.user)
 
         if self.use_rslib_model:
-            return self.FeatureUtil.feature_extraction(data=self.cur_rawstate, predict=True)
+            return self.FeatureUtil.feature_extraction(data=self.cur_rawstate, serving=True)
 
         observation = self.controllerobs.get_obs(self.cur_rawstate, self.info, self.stepp)
         if self.batch:
